import { _ as _defineProperty } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx, jsxs } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import classNames from 'classnames';
import exceptionIcon from '@jetbrains/icons/exception';
import checkmarkIcon from '@jetbrains/icons/checkmark';
import warningIcon from '@jetbrains/icons/warning';
import closeIcon from '@jetbrains/icons/close';
import Icon from '../icon/icon.js';
import LoaderInline from '../loader-inline/loader-inline.js';
import { getRect } from '../global/dom.js';
import joinDataTestAttributes from '../global/data-tests.js';
import { Button } from '../button/button.js';
import { T as Theme, a as ThemeProvider } from '../_helpers/theme.js';
export { default as Container } from './container.js';
import { Color } from '../icon/icon__constants.js';
import 'util-deprecate';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.split.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import '../link/clickableLink.js';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import 'react-dom';
import '../popup/popup.target.js';
import '../popup/popup.js';
import 'core-js/modules/es.regexp.to-string.js';
import '../global/get-uid.js';
import '../global/schedule-raf.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../tab-trap/tab-trap.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.consts.js';

var styles = {"alert":"alert_rui_2b32","alertInline":"alertInline_rui_2b32","error":"error_rui_2b32","icon":"icon_rui_2b32","caption":"caption_rui_2b32","withCloseButton":"withCloseButton_rui_2b32","badge":"badge_rui_2b32","loader":"loader_rui_2b32","close":"close_rui_2b32","animationOpen":"animationOpen_rui_2b32","show":"show_rui_2b32","animationClosing":"animationClosing_rui_2b32","animationShaking":"animationShaking_rui_2b32","shaking":"shaking_rui_2b32"};

const ANIMATION_TIME = 500;
/**
 * @name Alert
 */
/**
 * List of available alert types.
 * @enum {string}
 */
var AlertType;
(function (AlertType) {
  AlertType["ERROR"] = "error";
  AlertType["MESSAGE"] = "message";
  AlertType["SUCCESS"] = "success";
  AlertType["WARNING"] = "warning";
  AlertType["LOADING"] = "loading";
})(AlertType || (AlertType = {}));
/**
 * Lookup table of alert type to icon modifier.
 * @type {Object.<AlertType, string>}
 */
const TypeToIcon = {
  [AlertType.ERROR]: exceptionIcon,
  [AlertType.SUCCESS]: checkmarkIcon,
  [AlertType.WARNING]: warningIcon
};
/**
 * Lookup table of alert type to icon color.
 * @type {Object.<AlertType, Icon.Color>}
 */
const TypeToIconColor = {
  [AlertType.ERROR]: Color.RED,
  [AlertType.SUCCESS]: Color.GREEN,
  [AlertType.WARNING]: Color.WHITE
};
/**
 * @constructor
 * @name Alert
 * @extends {ReactComponent}
 */
/**
 * **Alert** is a component for displaying contextual notifications. If you want to display a stack of notifications, use **Alerts** instead.
 */
class Alert extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "state", {
      height: null
    });
    _defineProperty(this, "node", void 0);
    _defineProperty(this, "hideTimeout", void 0);
    _defineProperty(this, "closeRequest", event => {
      this.startCloseAnimation();
      return this.props.onCloseRequest(event);
    });
    _defineProperty(this, "startCloseAnimation", () => {
      const height = getRect(this.node).height;
      this.setState({
        height
      });
    });
    /**
     * @param {SyntheticEvent} evt
     * @private
     */
    _defineProperty(this, "_handleCaptionsLinksClick", evt => {
      if (evt.target instanceof Element && evt.target.matches('a')) {
        this.closeRequest(evt);
      }
    });
    _defineProperty(this, "storeAlertRef", node => {
      this.node = node;
    });
  }
  componentDidMount() {
    if (this.props.timeout > 0) {
      this.hideTimeout = window.setTimeout(this.closeRequest, this.props.timeout);
    }
  }
  componentDidUpdate() {
    if (this.props.isClosing) {
      this._close();
    }
  }
  componentWillUnmount() {
    clearTimeout(this.hideTimeout);
  }
  _close() {
    this.startCloseAnimation();
    setTimeout(() => {
      this.props.onClose();
    }, ANIMATION_TIME);
  }
  /**
   * @private
   */
  _getCaption() {
    return jsx("span", {
      className: classNames(styles.caption, this.props.captionClassName, {
        [styles.withCloseButton]: this.props.closeable
      }),
      onClick: this._handleCaptionsLinksClick,
      // We only process clicks on `a` elements, see above
      role: "presentation",
      children: this.props.children
    });
  }
  /**
   * @private
   * @return {XML|string}
   */
  _getIcon() {
    const glyph = TypeToIcon[this.props.type];
    if (glyph) {
      return jsx(Icon, {
        glyph: glyph,
        className: styles.icon,
        color: TypeToIconColor[this.props.type] || Color.DEFAULT
      });
    } else if (this.props.type === AlertType.LOADING) {
      return jsx(LoaderInline, {
        className: styles.loader
      });
    }
    return '';
  }
  render() {
    const {
      type,
      inline,
      isClosing,
      isShaking,
      closeButtonClassName,
      showWithAnimation,
      className,
      'data-test': dataTest,
      theme
    } = this.props;
    const classes = classNames(className, {
      [styles.alert]: true,
      [styles.animationOpen]: showWithAnimation,
      [styles.error]: type === 'error',
      [styles.alertInline]: inline,
      [styles.animationClosing]: isClosing,
      [styles.animationShaking]: isShaking
    });
    const height = this.state.height;
    const style = height ? {
      marginBottom: -height
    } : undefined;
    return jsxs(ThemeProvider, {
      theme: theme,
      className: classes,
      "data-test": joinDataTestAttributes('alert', dataTest),
      "data-test-type": type,
      style: style,
      ref: this.storeAlertRef,
      children: [this._getIcon(), this._getCaption(), this.props.closeable ? jsx(Button, {
        icon: closeIcon,
        className: classNames(styles.close, closeButtonClassName),
        "data-test": "alert-close",
        "aria-label": "close alert",
        onClick: this.closeRequest
      }) : '']
    });
  }
}
/** @override */
_defineProperty(Alert, "defaultProps", {
  theme: Theme.DARK,
  closeable: true,
  showWithAnimation: true,
  type: AlertType.MESSAGE,
  inline: true,
  isClosing: false,
  isShaking: false,
  timeout: 0,
  onClose: () => {},
  onCloseRequest: () => {}
});
_defineProperty(Alert, "Type", AlertType);

export { ANIMATION_TIME, AlertType, Alert as default };
