import { _ as _defineProperty } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.promise.js';
import 'core-js/modules/es.string.includes.js';
import ExtendableError from 'es6-error';
import { CODE } from '../http/http.js';
import 'core-js/modules/es.weak-map.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import '../global/url.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.match.js';
import 'core-js/modules/es.string.replace.js';

class TokenValidationError extends ExtendableError {
  constructor(message, cause) {
    super(message);
    _defineProperty(this, "cause", void 0);
    _defineProperty(this, "authRedirect", void 0);
    this.cause = cause;
    this.authRedirect = true;
  }
}
class TokenValidator {
  constructor(config, getUser, storage) {
    _defineProperty(this, "_getUser", void 0);
    _defineProperty(this, "_config", void 0);
    _defineProperty(this, "_storage", void 0);
    this._getUser = getUser;
    this._config = config;
    this._storage = storage;
  }
  /**
   * Returns epoch - seconds since 1970.
   * Used for calculation of expire times.
   * @return {number} epoch, seconds since 1970
   * @private
   */
  static _epoch() {
    const milliseconds = 1000.0;
    return Math.round(Date.now() / milliseconds);
  }
  /**
   * @const {number}
   */
  // eslint-disable-next-line @typescript-eslint/no-magic-numbers

  /**
   * Check token validity against all conditions.
   * @returns {Promise.<string>}
   */
  validateTokenLocally() {
    return this._getValidatedToken([TokenValidator._validateExistence, TokenValidator._validateExpiration, this._validateScopes.bind(this)]);
  }
  /**
   * Check token validity against all conditions.
   * @returns {Promise.<string>}
   */
  validateToken() {
    return this._getValidatedToken([TokenValidator._validateExistence, TokenValidator._validateExpiration, this._validateScopes.bind(this), this._validateAgainstUser.bind(this)]);
  }
  /**
   * Check if there is a token
   * @param {StoredToken} storedToken
   * @return {Promise.<StoredToken>}
   * @private
   */
  static _validateExistence(storedToken) {
    if (!storedToken || !storedToken.accessToken) {
      throw new TokenValidator.TokenValidationError('Token not found');
    }
  }
  /**
   * Check expiration
   * @param {StoredToken} storedToken
   * @return {Promise.<StoredToken>}
   * @private
   */
  static _validateExpiration(_ref) {
    let {
      expires,
      lifeTime
    } = _ref;
    const REFRESH_BEFORE_RATIO = 6;
    const refreshBefore = lifeTime ? Math.ceil(lifeTime / REFRESH_BEFORE_RATIO) : TokenValidator.DEFAULT_REFRESH_BEFORE;
    if (expires && expires < TokenValidator._epoch() + refreshBefore) {
      throw new TokenValidator.TokenValidationError('Token expired');
    }
  }
  /**
   * Check scopes
   * @param {StoredToken} storedToken
   * @return {Promise.<StoredToken>}
   * @private
   */
  _validateScopes(storedToken) {
    const {
      scope,
      optionalScopes
    } = this._config;
    const requiredScopes = optionalScopes ? scope.filter(scopeId => !optionalScopes.includes(scopeId)) : scope;
    const hasAllScopes = requiredScopes.every(scopeId => {
      var _storedToken$scopes;
      return (_storedToken$scopes = storedToken.scopes) === null || _storedToken$scopes === void 0 ? void 0 : _storedToken$scopes.includes(scopeId);
    });
    if (!hasAllScopes) {
      throw new TokenValidator.TokenValidationError("Token doesn't match required scopes");
    }
  }
  /**
   * Check by error code if token should be refreshed
   * @param {string} error
   * @return {boolean}
   */
  static shouldRefreshToken(error) {
    return error === 'invalid_grant' || error === 'invalid_request' || error === 'invalid_token';
  }
  /**
   * Check scopes
   * @param {StoredToken} storedToken
   * @return {Promise.<StoredToken>}
   * @private
   */
  async _validateAgainstUser(storedToken) {
    try {
      return await this._getUser(storedToken.accessToken);
      // eslint-disable-next-line @typescript-eslint/no-explicit-any
    } catch (errorResponse) {
      let response = {};
      try {
        response = await errorResponse.response.json();
      } catch (e) {
        // Skip JSON parsing errors
      }
      if (errorResponse.status === CODE.UNAUTHORIZED || TokenValidator.shouldRefreshToken(response.error)) {
        var _errorResponse$data, _errorResponse$data2;
        // Token expired
        throw new TokenValidator.TokenValidationError(response.error || errorResponse.message, (_errorResponse$data = errorResponse.data) !== null && _errorResponse$data !== void 0 && _errorResponse$data.error ? new Error((_errorResponse$data2 = errorResponse.data) === null || _errorResponse$data2 === void 0 ? void 0 : _errorResponse$data2.error) : undefined);
      }
      // Request unexpectedly failed
      throw errorResponse;
    }
  }
  /**
   * Token Validator function
   * @typedef {(function(StoredToken): Promise<StoredToken>)} TokenValidator
   */
  /**
   * Gets stored token and applies provided validators
   * @param {TokenValidator[]} validators An array of validation
   * functions to check the stored token against.
   * @return {Promise.<string>} promise that is resolved to access token if the stored token is valid. If it is
   * invalid then the promise is rejected. If invalid token should be re-requested then rejection object will
   * have {authRedirect: true}.
   * @private
   */
  async _getValidatedToken(validators) {
    const storedToken = await this._storage.getToken();
    if (storedToken == null) {
      throw new TokenValidator.TokenValidationError('Token not found');
    }
    for (let i = 0; i < validators.length; i++) {
      await validators[i](storedToken);
    }
    return storedToken.accessToken;
  }
}
_defineProperty(TokenValidator, "DEFAULT_REFRESH_BEFORE", 10 * 60);
// 20 min in s
/**
 * Error class for auth token validation
 *
 * @param {string} message Error message
 * @param {Error=} cause Error that caused this error
 */
_defineProperty(TokenValidator, "TokenValidationError", TokenValidationError);

export { TokenValidationError, TokenValidator as default };
